angular.module('Landers.app', [])
    .provider('AngularApp', function () {
        var provider = this;

        this.appPath = null;
        this.needCss = false;

        this.$get = ['$state', 'Ajax', 'Runtime', 'AngularModel', 'Dialog', '$templateRequest', 'Flat', function ($state, Ajax, Runtime, AngularModel, Dialog, $templateRequest, Flat) {
            function AngularApp(scope, config) {
                //参数纠正
                //key, name
                config = config || {};

                //初始化全局变量
                var that = this,
                    doms = [];

                this.scope = scope;
                scope.$app = scope.$app || {};
                scope.$app[config.key] = this;

                //初始化 key, name
                // this.key = config.key;
                // this.name = config.name;

                //初始化path
                this.path = provider.appPath + '/' + (config.path || config.key) + '/';

                //加载css
                if (provider.needCss) {
                    doms.push(Landers.loader.css(this.path + 'style.css'));
                }

                //初始化ajax
                this.ajax = Ajax.make();

                //初始化runtime (挂载$runtime至scope)
                this.runtime = Runtime.make(scope);

                this.$ = {
                    _datkey : function(objectKey, subkey){
                        var datkey = ['$' + objectKey];
                        if (subkey) datkey.push(subkey);
                        return datkey.join('.');
                    },
                    get : function(objectKey, subkey) {
                        var datkey = this._datkey(objectKey, subkey);
                        return Flat.get(scope, datkey);
                    },
                    set : function(objectKey, data, subkey) {
                        var datkey = this._datkey(objectKey);
                        Flat.set(scope, datkey, data, subkey);
                        if (!scope.$$phase) scope.$apply();
                    },
                    extend : function(objectKey, data) {
                        var datkey = this._datkey(objectKey);
                        Flat.extend(scope, data, datkey);
                        if (!scope.$$phase) scope.$apply();
                    }
                };

                //对话框
                this.dialog = {
                    dialogs : [],
                    _attachConfig : function(_dialog_config){
                        var dialog_config = angular.extend({}, _dialog_config);
                        if (_dialog_config.event) {
                            if (that.model) {
                                that.model.listitem.active(_dialog_config.event, true);
                            }
                            dialog_config.onclose = function(){
                                if (that.model) {
                                    that.model.listitem.active(_dialog_config.event, false);
                                }
                                _dialog_config.onclose && _dialog_config.onclose.call(this);
                            };
                            delete dialog_config.event;
                        }
                        dialog_config.blur = false;
                        return dialog_config;
                    },
                    _loadPartial : function(dialog_config, callback){
                        var partial = dialog_config.partial || dialog_config.tplkey;
                        var templateUrl = scope.$partial('dialog-' + partial);
                        $templateRequest(templateUrl).then(function(html){
                            callback(html);
                        });
                    },
                    show : function(dialog_config){
                        dialog_config = this._attachConfig(dialog_config);
                        var _this = this;
                        _this._loadPartial(dialog_config, function(html){
                            dialog_config.html = html;
                            var dialog = Dialog.show(scope, dialog_config);
                            _this.dialogs.push(dialog);
                        });
                    },
                    confirmWithPartial : function(dialog_config, callbacks){
                        dialog_config = this._attachConfig(dialog_config);
                        var _this = this;
                        _this._loadPartial(dialog_config, function(html){
                            dialog_config.html = html;
                            var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                            _this.dialogs.push(dialog);
                        });
                    },
                    // 传入要显示的内容和按钮的回调
                    confirm : function(content, callbacks, $event){
                        callbacks.yesFn = callbacks.yesFn || function () {};
                        callbacks.noFn = callbacks.noFn || function () {};

                        // 处理content内容
                        content = content.replace(/&/g, "&amp;")
                            .replace(/</g, "&lt;")
                            .replace(/>/g, "&gt;");

                        dialog_config = this._attachConfig({
                            content: '<div class=\"confirm\">' + content + '</div>',
                            title: '来自'+window.location.host+'的消息',
                            width: 300,
                            event: $event
                        });

                        var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                        this.dialogs.push(dialog);
                    }
                };

                //初始化model
                if (config.model !== false) {
                    var model_config = Landers.utils.json.slice(config, [
                        'key', 'name', 'api_url', 'url_style', 'url_params', 'ajax', 'vice'
                    ]);
                    model_config.ajax = model_config.ajax || that.ajax;
                    this.model = config.model || AngularModel.make(scope, model_config);

                    this.action = {
                        loading:function(){
                            if (that.ajax) that.ajax.loading();
                            return this;
                        },
                        post : function(action, data, params) {
                            var apiurl = that.model.api.buildUrl('custom', {action:action});
                            if (!data && that.model) data = that.model.getData('$edit');
                            if (that.ajax) return that.ajax.post(apiurl, data, params);
                        },
                        get : function(action, params) {
                            var apiurl = that.model.api.buildUrl('custom', {action:action});
                            if (that.ajax) return that.ajax.get(apiurl, params);
                        }
                    };
                    this.hooks = that.model.hooks;
                }

                //定义partial加载器
                scope.$partial = scope.$partial || function(partial, app_key) {
                    var path;
                    if (app_key === true) {
                        path = '/component';
                    } else {
                        app_key = app_key || (config.path || config.key);
                        path = provider.appPath + app_key;
                    }
                    return path + '/partials/' + partial + '.html';
                };

                //与内置默认$services合并
                scope.$service = scope.$service || {
                    tagsInput : function(fields){
                        if (!angular.isArray(fields)) fields = [fields];
                        angular.forEach(fields, function(item){
                            var data = that.model.getData('$edit');
                            that.runtime.set(item, {
                                data : data[item] ? data[item].split(',') : [],
                                onChange:function(){
                                    data[item] = this.data.map(function(item){
                                        return item.text;
                                    }).join(',');
                                }
                            });
                        });
                    }
                };

                if ( !config.vice ) {
                    //与内置$events合并
                    scope.$events = angular.extend(scope.$events || {}, {
                    });

                    //初始化
                    scope.$errors = {};

                    //scope销毁时事件
                    scope.$on('$destroy', function(){
                        //断开所有连接
                        that.ajax.cancelAll();
                        delete that.ajax;

                        //清除doms
                        angular.forEach(doms, function(dom){
                            dom.remove();
                        });

                        // Landers.dialog.clear && Landers.dialog.clear();
                        angular.forEach(that.dialog.dialogs, function(dialog){
                            if (dialog && dialog.close) {
                                dialog.close();
                            }
                        });

                        that.scope = null;

                        Landers.response.hide();
                    });
                }
            }

            return {
                make : function(scope, config){
                    return new AngularApp(scope, config);
                }
            };
        }];
    });